/****************************************************************
*
*	File: handle.h
*	Copyright(c) Andreas Magnusson 2002
*
*	This is my classes for reference counted objects.
*
*	DISCLAIMER AND STUFF:
*	This source-code is supplied AS IS. I cannot be held responsible for any damage caused
*	by use or misuse of this source-code nor any derivatives of it.
*	So use the code as you please, but give credit where they are due.
*
****************************************************************/
#ifndef __HANDLE_H__
#define __HANDLE_H__

class ref_counted
{
	int _ref_count;
public:
	ref_counted() : _ref_count(0) {}
	virtual ~ref_counted() {}
	int add_ref() {return ++_ref_count;}
	int release() {return --_ref_count;}
};

template<class T> class handle
{
	T *object;
	void release()
	{
		if(object)
		{
			if(object->release() == 0)
				delete object;
		}
	}
	void add_ref()
	{
		if(object)
			object->add_ref();
	}
	void assign(T *ptr)
	{
		if(ptr == object)
			return;
		release();
		object = ptr;
		add_ref();
	}
public:
	handle() : object(0) {}
	handle(T *pNew)
	{
		object = pNew;
		add_ref();
	}
	handle(const handle &hnd)
	{
		object = hnd.object;
		add_ref();
	}
	~handle()
	{
		release();
	}
	const handle &operator =(const handle &hnd)
	{
		assign(hnd.object);
		return *this;
	}
	const handle &operator =(T *pNew)
	{
		assign(pNew);
		return *this;
	}
	T *operator ->()
	{
		return object;
	}
	operator T *()
	{
		return object;
	}
	T &operator *()
	{
		return *object;
	}
	bool operator ==(const handle &hnd)
	{
		return object == hnd.object;
	}
	bool operator !=(const handle &hnd)
	{
		return object != hnd.object;
	}
	bool is_null()
	{
		return object == NULL;
	}
};


#endif

